@@ -22,12 +22,9 @@
 #   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 #   SOFTWARE.
 ###################################################################################
-
 import io
 import os
 import queue
-import random
-##############
 import shutil
 import threading
 import urllib.parse
@@ -39,12 +36,11 @@ from urllib.error import URLError, HTTPError
 
 import urllib3
 from eisenradio.api import ghettoApi
-import textwrap
 
 # logging.basicConfig(level=logging.DEBUG, format='[%(levelname)s] (%(threadName)-10s) %(message)s',)
 # logging.basicConfig(level=logging.INFO, format='[%(levelname)s] (%(threadName)-10s) %(message)s',)
 
-version = '0.8.30'
+version = '0.9.0'
 print(f'{version} ghettorecorder (eisen_radio module)')  # ########## version #############
 
 
@@ -69,16 +65,12 @@ class GBase:
 
     @staticmethod
     def make_directory(str_path):
-        access_rights = 0o755
         try:
-            os.mkdir(str_path, access_rights)
+            os.mkdir(str_path)
         except FileExistsError:
             pass
-            # print(' Folder exists: ' + str_path)
-            return False
         else:
             print('\t--> created directory: ' + str_path)
-            return True
 
     @staticmethod
     def remove_special_chars(str_name):
@@ -144,7 +136,7 @@ class GNet(GBase):
             return response.getcode()
 
     @staticmethod
-    def is_server_alive(url, key):
+    def is_server_alive(url, str_radio):
         # don't delete - urllib3 timeout=5, placebo, retries=None or =2, screw yourself, since half of conn. die
         # we have server up, but content not presented - zombie
         try:
@@ -152,17 +144,17 @@ class GNet(GBase):
         except HTTPError:  # lots of strange configured web server
 
             print('HTTPError')
-            GBase.dict_error[key] = 'HTTPError - check radio homepage'
+            GBase.dict_error[str_radio] = 'HTTPError - check radio homepage'
             return True
         except URLError as error:  # <urlopen error timed out>
-            print(f' ---> {key} server failed: {error} (no recording) {url}')
-            GBase.dict_error[key] = error
+            print(f' ---> {str_radio} server failed: {error} (no recording) {url}')
+            GBase.dict_error[str_radio] = error
             print('URLError')
             return False
         return True
 
     @staticmethod
-    def stream_filetype_url(url, key):
+    def stream_filetype_url(url, str_radio):
         try:
             with urllib.request.urlopen(url, timeout=15) as response:
                 headers = response.getheader('Content-Type')
@@ -184,10 +176,10 @@ class GNet(GBase):
         # application/x-winamp-playlist , audio/scpls , audio/x-scpls ,  audio/x-mpegurl
 
         try:
-            GIni.content_type[key] = headers
+            GIni.content_type[str_radio] = headers
             if len(headers) <= 5:
-                GIni.content_type[key] = 'audio/mpeg'
-            # print(f' stream_filetype_url: {GIni.content_type[key]}')
+                GIni.content_type[str_radio] = 'audio/mpeg'
+            # print(f' stream_filetype_url: {GIni.content_type[str_radio]}')
         except Exception as e:
             print(e)
             pass
@@ -196,44 +188,43 @@ class GNet(GBase):
 
 
 class GRecorder:
-    unknown_title_name = 'untitled_full_record_'
+    unknown_title_name = 'unknown_title'
     path_to_song_dict = {}  # {station : file path}
     current_song_dict = {}  # each thread writes the new title to the station key name {station : title}
     start_write_command = {}  # recorder head thread set command to copy first file
     search_pattern_found = {}
 
-    fifo_web_srv_queue = queue.Queue(maxsize=10)
-    fifo_dict = {}
     record_active_dict = {}
     listen_active_dict = {}
     ghetto_measure = {}
     ghettoApi.init_ghetto_measurements(ghetto_measure)
+    audio_stream_queue_dict = {}
+    ghettoApi.init_ghetto_audio_stream(audio_stream_queue_dict)
 
     @staticmethod  # copy keep-alive timeout=3000
-    def ghetto_recorder_display_title(url, ini_key):
-        # print(f' ghetto_recorder_display_title: {ini_key}')
-        update_terminal = ''.encode('utf-8')
-        stream_song_name = GRecorder.current_song_dict[ini_key]
+    def ghetto_recorder_display_title(url, str_radio):
+        # print(f' ghetto_recorder_display_title: {str_radio}')
+        update_terminal = ''
+        stream_song_name = GRecorder.current_song_dict[str_radio]
         GIni.fail_meta_dict[url] = 'False'  # message display or not
-        GRecorder.search_pattern_found[ini_key] = False
+        GRecorder.search_pattern_found[str_radio] = False
 
-        while not GBase.dict_exit[ini_key]:
+        while not GBase.dict_exit[str_radio]:
 
             if not len(stream_song_name) <= 2:
-                #  GRecorder.current_song_dict[ini_key] = stream_song_name  # WRITE SONG NAME   # :DEACTIVATED:
-                stream_song_name = GRecorder.current_song_dict[ini_key]  # :REVERSE:
+                #  GRecorder.current_song_dict[str_radio] = stream_song_name  # WRITE SONG NAME   # :DEACTIVATED:
+                stream_song_name = GRecorder.current_song_dict[str_radio]  # :REVERSE:
 
                 if not update_terminal == stream_song_name:
                     update_terminal = stream_song_name
                     try:
-                        print(f'\t\t {GBase.this_time().split()[1]} title on {ini_key}: \t {update_terminal}')
-                        # see what häppens, góod unìt têst case
+                        print(f'\t\t {GBase.this_time().split()[1]} title on {str_radio}: \t {update_terminal}')
                     except UnicodeEncodeError:
-                        print('\t\t title on ' + ini_key + ':\t' + 'unicode error (check "$ localectl status")')
-                        print('\t\t title on ' + ini_key + ':\t' + 'consider to add additional language support.')
+                        print('\t\t title on ' + str_radio + ':\t' + 'unicode error (check "$ localectl status")')
+                        print('\t\t title on ' + str_radio + ':\t' + 'consider to add additional language support.')
 
                     except Exception as ex:
-                        print('\t\t title on ' + ini_key + ':\t' + 'unknown error (print to your terminal)')
+                        print('\t\t title on ' + str_radio + ':\t' + 'unknown error (print to your terminal)')
                         print(ex)
                     else:
                         pass
@@ -242,35 +233,32 @@ class GRecorder:
 
             try:
                 for _ in GIni.search_title_keys_list:
-                    if _ == ini_key:
+                    if _ == str_radio:
                         # search option was checked,
                         # call - def search_pattern_start_record, search string match?
-                        got_it = GRecorder.search_pattern_start_record(stream_song_name, ini_key)
+                        got_it = GRecorder.search_pattern_start_record(stream_song_name, str_radio)
                         if got_it:
-                            GIni.start_stop_recording[ini_key] = 'start'
+                            GIni.start_stop_recording[str_radio] = 'start'
                         else:
-                            GIni.start_stop_recording[ini_key] = 'stop'
-                            GRecorder.search_pattern_found[ini_key] = False
+                            GIni.start_stop_recording[str_radio] = 'stop'
+                            GRecorder.search_pattern_found[str_radio] = False
 
             except Exception as ex:
                 print(ex)
 
             for sec in range(GBase.sleeper):
                 sleep(1)
-                if GBase.dict_exit[ini_key]:
-                    # print(f' stop get title {ini_key}')
+                if GBase.dict_exit[str_radio]:
+                    # print(f' stop get title {str_radio}')
                     break
-            # To Do: make (complete) fake headers to keep connection up
-            #        sync sleep time with recorder
-            #        one file record with name
 
     @staticmethod
-    def search_pattern_start_record(title, ini_key):
+    def search_pattern_start_record(title, str_radio):
         # search title for strings to see if we should start recording this
         search_list = []  # chop the string
 
         try:
-            strings = GIni.search_dict[ini_key]  # dict with user search strings
+            strings = GIni.search_dict[str_radio]  # dict with user search strings
             search_list = strings.encode('utf-8').lower().split(b' ')
         except KeyError:
             pass
@@ -280,15 +268,15 @@ class GRecorder:
             if not len(search_str) <= 1:  # b'' match problem
 
                 if not title.encode('utf-8').lower().find(search_str) == -1:  # find() returns -1, if not found
-                    if not GRecorder.search_pattern_found[ini_key]:
-                        print(f'<<<< match station: {ini_key} phrase: {search_str}')
-                        GRecorder.search_pattern_found[ini_key] = True
+                    if not GRecorder.search_pattern_found[str_radio]:
+                        print(f'<<<< match station: {str_radio} phrase: {search_str}')
+                        GRecorder.search_pattern_found[str_radio] = True
                     return True
         return False
 
     @staticmethod
     def ghetto_recorder_head(directory_save, stream_suffix, radio_short_key):
-        # target: tail works without brain until brain call interrupt
+        # target: tail works without brain
         # print(f' ghetto_recorder_head -- {radio_short_key}')
 
         sleep(GBase.sleeper)
@@ -308,7 +296,16 @@ class GRecorder:
             print('\t---> Recorder Write Failure in : ' + GBase.radio_base_dir)
             return
 
-        GBase.make_directory(GBase.radio_base_dir + '//' + radio_short_key)
+        try:
+            GBase.make_directory(GBase.radio_base_dir + '//' + radio_short_key)
+        except OSError:
+            # "heyth~!ere./\coolleagues?!Straße"
+            GRecorder.current_song_dict[radio_short_key] = '[{-_-}] ZZZzz  Recorder Failure !!! DIRECTORY !!!'
+            print('\t---> Recorder Write Failure in : ' + GBase.radio_base_dir + '//' + radio_short_key)
+            sleep(5)
+            GBase.dict_exit[radio_short_key] = True  # all radio_short_key treads-will-stop
+
+            return
 
         while not brake_loop:
             i = 0
@@ -334,7 +331,7 @@ class GRecorder:
 
                         first_record = False
                     else:
-                        fresh_file_path = directory_save + '//' + clean_name + stream_suffix
+                        fresh_file_path = directory_save + '//' + str(clean_name) + str(stream_suffix)
                         GRecorder.path_to_song_dict[radio_short_key] = fresh_file_path  # NEW PATH ... ... ...
 
                     if GBase.dict_exit[radio_short_key]:
@@ -355,10 +352,8 @@ class GRecorder:
                                 break
                             sleep(1)
 
-    @staticmethod  # org
-    def ghetto_recorder_tail(url, key, path_to_save, suffix):
-        # shall be as dump as possible
-
+    @staticmethod
+    def g_recorder_await_head(key):
         while not GBase.dict_exit[key]:
             try:
                 if GRecorder.start_write_command[key]:
@@ -368,212 +363,213 @@ class GRecorder:
             else:
                 sleep(.1)
 
-        stream_request_size = io.DEFAULT_BUFFER_SIZE
-        old_time = time()
-        request = ''
-        rand_short = random.randrange(4, 13, 3)
-
-        ghetto_recorder = path_to_save + '//__ghetto_recorder' + str(suffix)
+    @staticmethod
+    def g_recorder_check_dir(key):
+        # if dir could not be made, no value
         try:
-            ghetto_copy = GRecorder.path_to_song_dict[key]
+            file_write_path = GRecorder.path_to_song_dict[key]
+            return file_write_path
         except KeyError:
             GRecorder.current_song_dict[key] = '[{-_-}] ZZZzz zz z... Recorder Failure!'
             GBase.dict_exit[key] = True  # all key treads-will-stop
-            return
+            return False
+
+    @staticmethod
+    def g_recorder_request(url):
+        # url was tested before in Gnet
         try:
             request = GNet.http_pool.request('GET', url,
                                              headers={'Connection': 'keep-alive'},
                                              timeout=3000,
                                              preload_content=False)
+            return request
         except Exception as e:
             print(e)
-            pass
-
-        while not GBase.dict_exit[key]:
-            with open(ghetto_recorder, 'wb') as record_file:
-
-                for chunk in request.stream(stream_request_size):  # chunks to file
-
-                    # ############################################################## rec
-                    # print(f'rec {GRecorder.record_active_dict[key]}')
-                    # print(f'lis {GRecorder.listen_active_dict[key]}')
-                    if GRecorder.record_active_dict[key]:
-                        record_file.write(chunk)
-
-                    # ############################################################## rec
-                    if not ghetto_copy == GRecorder.path_to_song_dict[key]:
-
-                        try:
-                            if os.path.exists(ghetto_copy):
-                                os.remove(ghetto_copy)
-                            record_file.flush()
-
-                            ghetto_size = os.path.getsize(ghetto_recorder)
-                            if int(ghetto_size) >= int(stream_request_size):
-                                shutil.copyfile(ghetto_recorder, ghetto_copy)
-                        except Exception as ex:
-                            print(ex)
-                        else:
-                            record_file.truncate()
-                            record_file.seek(0)
-
-                    ghetto_copy = GRecorder.path_to_song_dict[key]  # SET NEW PATH (title) after copy
-                    # #################################
-                    if time() - old_time > rand_short:
-                        old_time = time()
-                        rh = request.headers
-                        rand_short = random.randrange(4, 12, 2)
-
-                    if not chunk:
-                        request = GNet.http_pool.request('GET', url,
-                                                         headers={'Connection': 'keep-alive'},
-                                                         timeout=3000,
-                                                         preload_content=False)
-                        rh = request.headers  # into oblivion
-                        break
-                    if GBase.dict_exit[key]:
-                        sleep(2)  # wait for last file name _incomplete_.....
-                        try:
-                            record_file.flush()
-
-                            ghetto_size = os.path.getsize(ghetto_recorder)
-                            # print(f' ghetto_recorder: {int(ghetto_size)}')
-                            # print(f' stream_request_size): {int(stream_request_size)}')
-
-                            if int(ghetto_size) >= int(stream_request_size):
-                                ghetto_copy = GRecorder.path_to_song_dict[key]  # full path last file name
-                                print(f'\t "{key}" last file (marked: _incomplete_): '
-                                      f'{GRecorder.current_song_dict[key]} ')
-
-                                shutil.copyfile(ghetto_recorder, ghetto_copy)
-                            record_file.close()
-
-                            if os.path.exists(ghetto_recorder):
-                                os.remove(ghetto_recorder)
-                        except UnicodeEncodeError:
-                            print(f' {key} copy last file (marked: _incomplete_):')
-                        except Exception as ex:
-                            print(ex)
-                        break
-            if GBase.dict_exit[key]:
-                # print(f' exit file record {key}')
-                break
+            return False
 
     @staticmethod
-    def get_metadata_from_stream_loop(url, key):
-        # urllib3 request.stream with headers={'Icy-MetaData': '1'} , keep open (mdr, swr, br5) test
-
-        response = ''
-        request = ''
-        title = ''
-        metadata_content = ''
-        old_time = time()
-        rand_short = random.randrange(4, 12, 2)
-        rand_long = random.randrange(50, 80, 1)
-
-        while not GBase.dict_exit[key]:
+    def g_recorder_next_file(str_radio, full_file_path, record_file, ghetto_recorder, stream_request_size):
+        # COMPLETE file path has changed, new file to write; got new metadata
+        if not full_file_path == GRecorder.path_to_song_dict[str_radio]:
             try:
-                response = GNet.http_pool.request('GET', url,
-                                                  headers={'Connection': 'keep-alive'},
-                                                  timeout=3000, preload_content=False)
-            except Exception as e:
-                print(e)
+                if os.path.exists(full_file_path):
+                    os.remove(full_file_path)
+                record_file.flush()
+
+                ghetto_size = os.path.getsize(ghetto_recorder)
+                if int(ghetto_size) >= int(stream_request_size):
+                    shutil.copyfile(ghetto_recorder, full_file_path)
+            except Exception as ex:
+                print(ex)
+
+            record_file.seek(0)
+            record_file.truncate()
+
+        full_file_path = GRecorder.path_to_song_dict[str_radio]  # SET NEW PATH (title) after copy
+        return full_file_path
+
+    @staticmethod
+    def g_recorder_teardown(str_radio, record_file, ghetto_recorder, stream_request_size):
+        try:
+            record_file.flush()
+            ghetto_size = os.path.getsize(ghetto_recorder)
+
+            if int(ghetto_size) >= int(stream_request_size):
+                ghetto_copy = GRecorder.path_to_song_dict[str_radio]  # full path last file name
+                print(f'\t "{str_radio}" last file (marked: _incomplete_): '
+                      f'{GRecorder.current_song_dict[str_radio]} ')
+                shutil.copyfile(ghetto_recorder, ghetto_copy)
+            record_file.close()
+
+            if os.path.exists(ghetto_recorder):
+                os.remove(ghetto_recorder)
+        except UnicodeEncodeError:
+            print(f' {str_radio} copy last file (marked: _incomplete_):')
+        except OSError as ex:
+            print(ex)
+
+    @staticmethod
+    def g_recorder_write_queue(str_radio, chunk):
+        if not GRecorder.audio_stream_queue_dict[str_radio + ',audio'].full():
+            GRecorder.audio_stream_queue_dict[str_radio + ',audio'].put(chunk)
+
+    @staticmethod
+    def ghetto_recorder_tail(url, str_radio, path_to_save, suffix, str_action):
+
+        stream_request_size = io.DEFAULT_BUFFER_SIZE * 4
+        audio_stream_queue = queue.Queue(maxsize=20)  # 8KB * 4 * 20
+        GRecorder.audio_stream_queue_dict[str_radio + ',audio'] = audio_stream_queue
+        ghetto_recorder = path_to_save + '//__ghetto_recorder' + str(suffix)
+
+        GRecorder.g_recorder_await_head(str_radio)
+
+        # for thread in threading.enumerate():
+        #     print(thread.name)
+
+        full_file_path = GRecorder.g_recorder_check_dir(str_radio)
+        if full_file_path:
+            request = GRecorder.g_recorder_request(url)
+
+            if str_action == "record":
+                with open(ghetto_recorder, 'wb') as record_file:
+                    for chunk in request.stream(stream_request_size):
+                        record_file.write(chunk)
+                        full_file_path = GRecorder.g_recorder_next_file(str_radio,
+                                                                        full_file_path,
+                                                                        record_file,
+                                                                        ghetto_recorder,
+                                                                        stream_request_size)
+
+                        if not GRecorder.record_active_dict[str_radio]:
+                            GRecorder.g_recorder_teardown(str_radio, record_file, ghetto_recorder, stream_request_size)
+                            break
+
+            if str_action == "listen":
+                for chunk in request.stream(stream_request_size):
+                    GRecorder.g_recorder_write_queue(str_radio, chunk)
+                    if not GRecorder.listen_active_dict[str_radio]:
+                        break
+
+    @staticmethod
+    def metadata_header_info(request, str_radio, request_time):
+        try:
+            GRecorder.ghetto_measure[str_radio + ',request_time'] = request_time
+            GRecorder.ghetto_measure[str_radio + ',suffix'] = request.headers['content-type']
+            GRecorder.ghetto_measure[str_radio + ",icy_genre"] = request.headers["icy-genre"]
+            GRecorder.ghetto_measure[str_radio + ",icy_name"] = request.headers["icy-name"]
+            GRecorder.ghetto_measure[str_radio + ",icy_br"] = request.headers["icy-br"]
+        except KeyError:
+            pass
+
+    @staticmethod
+    def metadata_icy_info(request):
+        try:
+            icy_metadata = request.headers['icy-metaint']
+            icy_metadata = int(icy_metadata)
+
+            request.read(icy_metadata)
+            chunk_1b = request.read(1)
+            chunk_1b = ord(chunk_1b)
+            read_bytes = chunk_1b * 16
+            read_bytes = int(read_bytes)
+            metadata_content = request.read(read_bytes)
+            return metadata_content
+        except KeyError:
+            return False
+
+    @staticmethod
+    def metadata_request(url):
+
+        request = GNet.http_pool.request('GET', url,
+                                         headers={'Icy-MetaData': '1'},
+                                         preload_content=False)
+        return request
+
+    @staticmethod
+    def metadata_get_display_extract(icy_info):
+        # StreamTitle='X-Dream - Panic In Paradise * anima.sknt.ru';StreamUrl='';
+        try:
+            title_info = icy_info.split(";")
+            if not len(title_info) > 1 or title_info is None:
+                return    # empty list
+            title_info = title_info[0].split("=")
+            title = str(title_info[1])
+            title = GBase.remove_special_chars(title)
+            return title
+        except IndexError:
+            pass
+        except OSError:
+            pass
+
+    @staticmethod
+    def metadata_get_display_info(icy_info):
+        # <class 'bytes'> decode to <class 'str'>
+        title = ''
+        try:
+            title = GRecorder.metadata_get_display_extract(icy_info.decode('utf-8'))
+        except AttributeError:
+            pass
+        if not title:
+            return
+        if title:
+            return title
+
+    @staticmethod
+    def get_metadata_from_stream_loop(url, str_radio):
+        icy_info = ''
+        while not GBase.dict_exit[str_radio]:
+            try:
+                start_time = time()
+                response = GRecorder.metadata_request(url)
+                request_time = round((time() - start_time) * 1000)
+                GRecorder.metadata_header_info(response, str_radio, request_time)
+                icy_info = GRecorder.metadata_icy_info(response)
+            except HTTPError:
+                pass
+            except URLError:
+                """url was checked in is_server_alive. assume short conn error"""
                 pass
 
-            while not GBase.dict_exit[key]:
-
-                if time() - old_time > rand_short:
-                    old_time = time()
-                    rh = response.headers  # keep conn. alive
-                    rand_short = random.randrange(4, 12, 2)
-                    # print(response.headers)
-
-                if time() - old_time > rand_long:  # new connection
-                    old_time = time()
-                    rand_long = random.randrange(50, 80, 1)
-                    break
+            title = GRecorder.metadata_get_display_info(icy_info)
+            if title:
                 try:
-                    start_time = time()
-                    request = GNet.http_pool.request('GET', url,
-                                                     headers={'Icy-MetaData': '1'},
-                                                     preload_content=False)
-                    try:
-                        GRecorder.ghetto_measure[key + ',request_time'] = round((time() - start_time) * 1000)
-                        GRecorder.ghetto_measure[key + ',suffix'] = request.headers['content-type']
-                    except KeyError:
-                        pass
-                    try:
-                        icy_genre = request.headers["icy-genre"]  # [0:30]
-                        icy_name = request.headers["icy-name"]
-                        icy_br = request.headers["icy-br"]
-                        GRecorder.ghetto_measure[key + ",icy_genre"] = icy_genre
-                        GRecorder.ghetto_measure[key + ",icy_name"] = icy_name
-                        GRecorder.ghetto_measure[key + ",icy_br"] = icy_br
-                    except KeyError:
-                        pass
-                    # print(request.headers)
+                    if title[0] == "'" and title[-1] == "'":
+                        GRecorder.current_song_dict[str_radio] = title[1:-1]   # sequence drop 0 and last char
+                    else:
+                        GRecorder.current_song_dict[str_radio] = title
+                except KeyError:
+                    pass
                 except Exception as e:
                     print(e)
                     pass
 
-                try:
-                    pass
-
-                except KeyError:
-                    pass
-                try:
-                    icy_metadata = request.headers['icy-metaint']
-                    icy_metadata = int(icy_metadata)
-
-                    request.read(icy_metadata)
-                    chunk_1b = request.read(1)
-                    chunk_1b = ord(chunk_1b)
-                    read_bytes = chunk_1b * 16
-                    read_bytes = int(read_bytes)
-                    metadata_content = request.read(read_bytes)
-                except KeyError:
-                    pass
-                try:
-                    metadata_content = metadata_content.decode('utf-8')
-                    title_info = metadata_content.split(";")
-                    title_info = title_info[0].split("=")
-                    title = str(title_info[1])
-                    title = GBase.remove_special_chars(title)
-
-                    request.release_conn()  # REQUEST one shot anyway
-                    # print(f' -> {key} + {title}')
-                except IndexError:
-                    try:
-                        metadata_content = metadata_content
-                        title_info = metadata_content.split(";")
-                        title_info = title_info[0].split("=")
-                        title = str(title_info[1])
-                        title = GBase.remove_special_chars(title)
-                    except Exception:
-                        request.release_conn()  # REQUEST one shot anyway
-                        pass
-
-                else:
-                    try:
-                        if title[0] == "'" and title[-1] == "'":
-                            GRecorder.current_song_dict[key] = title[1:-1]
-                        else:
-                            GRecorder.current_song_dict[key] = title
-                        # print(f' -> {key} + {GRecorder.current_song_dict[key]}')
-                    except KeyError:
-                        pass
-                    except Exception as e:
-                        print(e)
-                        pass
-                    else:
-                        for sec in range(rand_short):
-                            if GBase.dict_exit[key]:
-                                # print(f' out reader {key}')
-                                break
-                            sleep(1)
+            for sec in range(2):
+                if GBase.dict_exit[str_radio]:
+                    break     # while
+                sleep(1)
 
     @staticmethod
-    def playlist_m3u(url, key):
+    def playlist_m3u(url, str_radio):
         # returns the first server of the playlist (not only m3u)
         try:
             read_url = GNet.http_pool.request('GET', url, preload_content=False)
@@ -592,7 +588,7 @@ class GRecorder:
                     # print(len(m3u_streams))
 
             if len(m3u_streams) > 1:
-                print(f' {key} Have more than one server in playlist_m3u. !!! Take first stream available.')
+                print(f' {str_radio} Have more than one server in playlist_m3u. !!! Take first stream available.')
                 play_server = m3u_streams[0]
                 return play_server
             if len(m3u_streams) == 1:
@@ -604,43 +600,43 @@ class GRecorder:
                 return False
 
 
-def check_alive_playlist_container(str_key, str_url):
+def check_alive_playlist_container(str_radio, str_url):
     # playlist url?
     if str_url[-4:] == '.m3u' or str_url[-4:] == '.pls':  # or url[-5:] == '.m3u8' or url[-5:] == '.xspf':
         # take first from the list
-        is_playlist_server = GRecorder.playlist_m3u(str_url, str_key)
+        is_playlist_server = GRecorder.playlist_m3u(str_url, str_radio)
 
         if not is_playlist_server == '':
-            if GNet.is_server_alive(str_url, str_key):
+            if GNet.is_server_alive(str_url, str_radio):
                 str_url = is_playlist_server
             else:
                 print('   --> playlist_server server failed, no recording')
             return str_url
     else:
-        GNet.is_server_alive(str_url, str_key)
+        GNet.is_server_alive(str_url, str_radio)
         return False
 
 
-def record(ini_key, url):
-    stream_suffix = GNet.stream_filetype_url(url, ini_key)  # ret and write in content-type dict
-    GRecorder.current_song_dict[ini_key] = GRecorder.unknown_title_name
-    GIni.start_stop_recording[ini_key] = 'start'
-    # GIni.start_stop_recording[ini_key] = 'stop'            # init it here, should be set via user interface
-    GIni.start_stop_recording[ini_key + '_adv'] = 'start_from_here'  # MUST be set or a key error in record def:
+def record(str_radio, url, str_action):
+    stream_suffix = GNet.stream_filetype_url(url, str_radio)
+    GRecorder.current_song_dict[str_radio] = GRecorder.unknown_title_name
+    GIni.start_stop_recording[str_radio] = 'start'  # remnant of hit keyword, start record in tk/tcl g.recorder
+    # GIni.start_stop_recording[str_radio] = 'stop'   # init it here, checkbox user interface
+    GIni.start_stop_recording[str_radio + '_adv'] = 'start_from_here'
 
-    dir_save = GBase.radio_base_dir + '//' + ini_key
+    dir_save = GBase.radio_base_dir + '//' + str_radio
 
-    threading.Thread(target=GRecorder.ghetto_recorder_display_title,
-                     args=(url, ini_key),
+    threading.Thread(name=str_radio + '_' + str_action + "_show", target=GRecorder.ghetto_recorder_display_title,
+                     args=(url, str_radio),
                      daemon=True).start()
-    threading.Thread(target=GRecorder.ghetto_recorder_head,
-                     args=(dir_save, stream_suffix, ini_key),
+    threading.Thread(name=str_radio + '_' + str_action + "_head", target=GRecorder.ghetto_recorder_head,
+                     args=(dir_save, stream_suffix, str_radio),
                      daemon=True).start()
-    threading.Thread(target=GRecorder.ghetto_recorder_tail,
-                     args=(url, ini_key, dir_save, stream_suffix),
+    threading.Thread(name=str_radio + '_' + str_action + "_tail", target=GRecorder.ghetto_recorder_tail,
+                     args=(url, str_radio, dir_save, stream_suffix, str_action),
                      daemon=True).start()
-    threading.Thread(target=GRecorder.get_metadata_from_stream_loop,
-                     args=(url, ini_key),
+    threading.Thread(name=str_radio + '_' + str_action + "_meta", target=GRecorder.get_metadata_from_stream_loop,
+                     args=(url, str_radio),
                      daemon=True).start()
 
     # ################################## end ########################################
